#include <UrsUtil.h>

URS::UtilClass& UrsUtil = URS::Util;
namespace URS {
   UtilClass Util;

   String UtilClass::thousandSeparatorString(uint32_t n) const {
      char retbuf[15];
      char* p = &retbuf[sizeof(retbuf) - 1];
      int i = 0;

      *p = '\0';

      do {
         if (i % 3 == 0 && i != 0)
            *--p = thousandSeparator;
         *--p = '0' + n % 10;
         n /= 10;
         i++;
      } while (n != 0);

      return p;
   }

   String UtilClass::thousandSeparatorString(int32_t n) const {
      if (n < 0) {
         return "-" + thousandSeparatorString((uint32_t)(-n));
      }
      return thousandSeparatorString((uint32_t)n);
   }

   static String toHexInternal(uint32_t n, uint8_t digits, const char separator = '\0') {
      char retbuf[10];
      if (digits > 8) digits = 8;    // Keine unzulssige Anzahl an Stellen
      char* p = retbuf + digits;    // Position hinte der letzten Stelle
      if (separator != '\0') p++;    // Platz fr Separator
      *(p--) = '\0';                 // String abschlieen
      for (int i = 0; i < digits; i++) {
         uint8_t q = n & 0x0F;
         if (i == 4 && separator != '\0')
            *(p--) = separator;
         *(p--) = q > 9 ? q + 'A' - 10 : q + '0';
         n = n >> 4;
      }

      return retbuf;
   }

   String UtilClass::toHex2Digits(uint8_t n) const {
      return toHexInternal(n, 2);
   }

   String UtilClass::toHex4Digits(uint16_t n) const {
      return toHexInternal(n, 4);
   }

   String UtilClass::toHex8Digits(uint32_t n, const char separator) const {
      return toHexInternal(n, 8, separator);
   }

   String UtilClass::tail(const String& source, const size_t length) const {
      if (length >= source.length()) { return source; }
      return source.substring(source.length() - length);
   } // tail

   void UtilClass::reverse(char* src) const {
      int i = 0;
      int j = strlen(src) - 1; // counting the length

      while (i < j) {  // for reversing string
         char temp = src[i];
         src[i] = src[j];
         src[j] = temp;
         i++;
         j--;
      }
   }

   String UtilClass::reverse(const String& src) const {
      String s = src;
      reverse((char*)(s.c_str()));
      return s;
   }

   String URS::UtilClass::toBin32Digits(const uint32_t n, const char separator) const {
      String q(n, 2);
      q = String("00000000000000000000000000000000") + q;
      q = tail(q, 32);
      if (separator != '\0') {
         q = q.substring(0, 4) + separator + q.substring(4, 8) + separator
            + q.substring(8, 12) + separator + q.substring(12, 16) + separator
            + q.substring(16, 20) + separator + q.substring(20, 24) + separator
            + q.substring(24, 28) + separator + q.substring(28, 32);
      }
      return q;
   }

   String URS::UtilClass::toBin16Digits(const uint16_t n, const char separator) const {
      String q(n, 2);
      q = String("0000000000000000") + q;
      q = tail(q, 16);
      if (separator != '\0') {
         q = q.substring(0, 4) + separator + q.substring(4, 8) + separator
            + q.substring(8, 12) + separator + q.substring(12, 16);
      }
      return q;
   }

   String URS::UtilClass::toBin8Digits(const uint8_t n, const char separator) const {
      String q(n, 2);
      q = String("00000000") + q;
      q = tail(q, 8);
      if (separator != '\0') {
         q = q.substring(0, 4) + separator + q.substring(4, 8);
      }
      return q;
   }

   String URS::UtilClass::getCppVersion() const {
      if (__cplusplus == 202002L) return "C++20";
      else if (__cplusplus == 201703L) return "C++17";
      else if (__cplusplus == 201402L)return "C++14";
      else if (__cplusplus == 201103L) return "C++11";
      else if (__cplusplus == 199711L) return "C++98";
      else return "pre-standard C++";
   }
}